'use strict';

import { register, IConfirmationService, IMessageService, COMMON_EVENT } from "@systemorph/web";
import { getAddGroupAssignmentTitleTextForGroup, getAddGroupAssignmentTitleTextForAccessObject } from "./utils/helpers";
import { RequestForAccessObjectDialogController, AddToGroupDialogController, CompleteAccessReviewDialogController, CreateAssignmentsSummaryDialogController} from "./components";
import { IPromise, blockUI, IQService, IScope } from 'angular';
import {
    IAccessControlDialogService,
    IAccessControlService,
    IGroupAssignmentService,
    IGroupMetaInfo,
    IAccessReviewService,
    IAddGroupAssignmentsIntoGroupDialogResult,
    IRequestGroupAssignmentsForAccessObjectDialogResult,
    ICreateGroupAssignmentSummaryResult
} from "./accessControl.api";
import { isEmpty } from "lodash";

class AccessControlDialogService implements IAccessControlDialogService {
    constructor(private accessControlService: IAccessControlService,
                private $rootScope: IScope,
                private groupAssignmentService: IGroupAssignmentService,
                private accessReviewService: IAccessReviewService,
                private $q: IQService,
                private confirmationService: IConfirmationService,
                private messageService: IMessageService,
                private blockUI: blockUI.IBlockUIService) {
    }

    addOrRequestGroupAssignmentIntoGroup(groupMetaInfo: IGroupMetaInfo): IPromise<any> {
        this.blockUI.start("Loading...");
        return this.$q.all([
            this.accessControlService.getAccessObjectToBeAddedInGroup(groupMetaInfo.systemName, groupMetaInfo.assignmentType),
            this.accessControlService.getAllGroupScopeTypeInfosDictionaryCached()
        ]).then(([accessObjects, groupScopeTypeInfosDictionary]) => {
            this.blockUI.stop();
            return this.confirmationService.open({
                cancelLabel: "Cancel",
                okLabel: "Ok",
                okCssClass: 'btn-success',
                size: "lg",
                title: getAddGroupAssignmentTitleTextForGroup(groupMetaInfo.assignmentType, groupMetaInfo.isRequestedGroup, groupMetaInfo.displayName),
                template: require('./components/dialogs/addToGroupDialog.html'),
                controller: AddToGroupDialogController,
                controllerAs: "addCtrl",
                resolve: {
                    groupMetaInfo: () => {
                        return groupMetaInfo;
                    },
                    accessObjects: () => {
                        return accessObjects;
                    },
                    groupScopeTypeInfosDictionary: () => {
                        return groupScopeTypeInfosDictionary;
                    }
                }
            }).then((result: IAddGroupAssignmentsIntoGroupDialogResult) => {
                this.blockUI.start("Saving...");
                return this.groupAssignmentService.createGroupAssignments([groupMetaInfo.systemName],
                                                                          groupMetaInfo.assignmentType,
                                                                          result.accessObjectSystemNames,
                                                                          result.scopeIds || null)
                    .then(summary => {
                        if (this.shouldShowCreateSummary(summary)){
                            this.showCreateSummary(summary);
                        }
                    })
                    .catch(ret => {
                        const message = `Error on assigning: ${ret.data.message}.`;
                        this.messageService.alertError(message);
                        throw message;
                    })
                    .finally(() => {
                        this.blockUI.stop();
                    });
            });
        })
    }

    requestGroupAssignmentForAccessObject(assignmentType: string, accessObjectSystemName: string, accessObjectDisplayName: string): IPromise<any> {
        this.blockUI.start("Loading...");
        return this.$q.all([
            this.accessControlService.getGroupsToBeRequested(accessObjectSystemName, assignmentType),
            this.accessControlService.getAllGroupScopeTypeInfosDictionaryCached(),
        ]).then(([groups, groupScopeTypeInfosDictionary]) => {
            this.blockUI.stop();
            return this.confirmationService.open({
                cancelLabel: "Cancel",
                okLabel: "Ok",
                okCssClass: 'btn-success',
                size: "lg",
                title: getAddGroupAssignmentTitleTextForAccessObject(assignmentType, accessObjectDisplayName),
                template: require('./components/dialogs/requestForAccessObjectDialog.html'),
                controller: RequestForAccessObjectDialogController,
                controllerAs: "addCtrl",
                resolve: {
                    assignmentType: () => {
                        return assignmentType;
                    },
                    groups: () => {
                        return groups;
                    },
                    groupScopeTypeInfosDictionary: () => {
                        return groupScopeTypeInfosDictionary;
                    }
                }
            }).then((result: IRequestGroupAssignmentsForAccessObjectDialogResult) => {
                this.blockUI.start("Saving...");
                return this.groupAssignmentService.createGroupAssignments(result.groupSystemNames,
                                                                          assignmentType,
                                                                          [accessObjectSystemName],
                                                                          result.scopeIds || null)
                    .then(summary => {
                        if (this.shouldShowCreateSummary(summary)){
                            this.showCreateSummary(summary);
                        }
                    })
                    .catch(ret => {
                        const message = `Error on requesting: ${ret.data.message}.`;
                        this.messageService.alertError(message);
                        throw message;
                    })
                    .finally(() => {
                        this.blockUI.stop();
                    });
            });
        })
    }

    private shouldShowCreateSummary(summary: ICreateGroupAssignmentSummaryResult): boolean {
        return !isEmpty(summary.errorMessages) || !isEmpty(summary.duplicateMessages)
    }

    private showCreateSummary(summary: ICreateGroupAssignmentSummaryResult) {
        return this.confirmationService.open({
            hideCancel: true,
            okLabel: "Ok",
            okCssClass: 'btn-default',
            size: "lg",
            title: "Group assignment summary",
            template: require('./components/dialogs/createAssignmentsSummaryDialog.html'),
            controller: CreateAssignmentsSummaryDialogController,
            controllerAs: "ctrl",
            resolve: {
                summary: () => {
                    return summary;
                }
            }
        });
    }
        
    completeReview(reviewId: string, initialComment: string): IPromise<any> {
        return this.confirmationService.open({
            cancelLabel: "Cancel",
            okLabel: "Sign & close",
            okCssClass: 'btn-success',
            size: "lg",
            title: "Complete access review",
            template: require('./components/dialogs/completeAccessReviewDialog.html'),
            controller: CompleteAccessReviewDialogController,
            controllerAs: "ctrl",
            resolve: {
                initialComment: () => {
                    return initialComment;
                }
            }
        }).then((comment: string) => {
            this.blockUI.start("Saving...");
            return this.accessReviewService.completeReview(reviewId, comment)
                .then(() => {
                    this.$rootScope.$broadcast(COMMON_EVENT.commitDataToServer);
                })
                .finally(() => {
                    this.blockUI.stop();
                });
        });
    }
}

register.service("accessControlDialogService", AccessControlDialogService)