'use strict';

import {
    register,
    ILayoutService,
    IPersisterProviderService,
    ISystemorphEntity,
    IEntityService, 
    getOrAddCachedValue,
    DISPLAY_CONTEXT,
    arrayToDictionary
} from "@systemorph/web";
import { IPromise, IHttpService, ICacheFactoryService, ICacheObject } from 'angular';
import { Dictionary } from 'lodash';
import {
    IAccessControlService,
    IGroupMetaInfo,
    IGroupAssignmentActivityModel,
    IGroupScopeTypeInfoModel
} from "./accessControl.api";
import { AC_TYPE_NAME, AC_QUERY_CONFIGURATION } from './accessControl.constants';

class AccessControlService implements IAccessControlService {
    private cache: ICacheObject;
    private readonly baseApiUrl: string = '/api/accessControlUtils/';

    constructor(private $http: IHttpService,
        $cacheFactory: ICacheFactoryService,
        private persisterProviderService: IPersisterProviderService,
        private layoutService: ILayoutService,
        private entityService: IEntityService) {
        
        this.cache = $cacheFactory("accessControlCache");
    }

    getSingleAccessObject(accessObjectId: string): IPromise<ISystemorphEntity> {
        return this.entityService.entityInfo(accessObjectId).then(entityInfo => {
            return this.layoutService.layout(entityInfo.typeName, DISPLAY_CONTEXT.details).$promise.then(layout => {
                return this.persisterProviderService.query(layout.collectionName).then(query => {
                    return query.find(accessObjectId, true).then(e => {
                        return e;
                    })
                })
            })
        })
    }

    getGroupMetaInfo(groupId: string, assignmentType: string): IPromise<IGroupMetaInfo> {
        const params = {
            groupId: groupId,
            assignmentType: assignmentType
        };
        return this.$http.get<IGroupMetaInfo>(`${this.baseApiUrl}groupMetaInfo`, { params: params })
            .then(result => result.data);
    }

    getAccessObjectToBeAddedInGroup(groupSystemName: string, assignmentType: string): IPromise<ISystemorphEntity[]> {
        return this.layoutService.layout(AC_TYPE_NAME.accessObject, DISPLAY_CONTEXT.details).$promise.then(layout => {
            return this.persisterProviderService.query(layout.collectionName).then(query => {
                return query.withQueryContext(AC_QUERY_CONFIGURATION.accessObjectsToBeAdded, { groupSystemName: groupSystemName, assignmentType: assignmentType }).toArray(true);
            })
        })
    }

    getGroupsToBeRequested(accessObjectSystemName: string, assignmentType: string): IPromise<ISystemorphEntity[]> {
        return this.layoutService.layout(AC_TYPE_NAME.group, DISPLAY_CONTEXT.details).$promise.then(layout => {
            return this.persisterProviderService.query(layout.collectionName).then(query => {
                return query.withQueryContext(AC_QUERY_CONFIGURATION.groupsToBeRequested, { accessObjectSystemName: encodeURIComponent(accessObjectSystemName) , assignmentType: assignmentType }).toArray(true);
            })
        })
    }

    getAllGroupAssignmentActivitiesDictionaryCached(): IPromise<Dictionary<IGroupAssignmentActivityModel>> {
        return getOrAddCachedValue(this.cache, "AllGroupAssignmentActivities", () => {
            return this.$http.get<IGroupAssignmentActivityModel[]>(`${this.baseApiUrl}AllGroupAssignmentActivities`).then(result => {
                return arrayToDictionary(result.data, x => x.value);
            })
        });
    }

    getAllGroupScopeTypeInfosDictionaryCached(): IPromise<Dictionary<IGroupScopeTypeInfoModel>> {
        return getOrAddCachedValue(this.cache, "AllGroupScopeTypeInfos", () => {
            return this.$http.get<IGroupScopeTypeInfoModel[]>(`${this.baseApiUrl}AllGroupScopeTypeInfos`).then(result => {
                return arrayToDictionary(result.data, x => x.name);
            })
        });
    }
}

register.service("accessControlService", AccessControlService)