'use strict';

import { register, IMessageService, IConfirmationService, DATE_TIME_KIND, DateTimeRangeModel } from '@systemorph/web';
import { IGroupAssignmentModel, IGroupAssignmentService, IRequestedGroupAssignmentsUiFilter } from '../../accessControl.api';
import { IGroupScopeFilterModel } from "../filters";
import { IPromise, IScope, blockUI, IFormController, IAugmentedJQuery } from 'angular';

interface ISelectableRequestedGroupAssignmentModel extends IGroupAssignmentModel {
    isSelected?: boolean;
}

register.directive('requestsApproval', () => {
    return {
        replace: true,
        scope: true,
        restrict: 'E',
        template: require('./requestsApproval.html'),
        controller: RequestsApprovalController,
        controllerAs: 'artbaCtrl',
        bindToController: {
        }
    }
});

interface IFilterModel {
    scope: IGroupScopeFilterModel;
    timeRange: DateTimeRangeModel;
    issuersSubstring: string;
    membersSubstring: string;
    groupsSubstring: string;
    assignmentType?: string;
}

class RequestsApprovalController {
    blockName: string;
    filterModel: IFilterModel;
    filteredItems: ISelectableRequestedGroupAssignmentModel[];

    filterForm: IFormController;

    readonly dateTimeKind: string = DATE_TIME_KIND.utc;
    readonly dateTimeFormat: string = 'short';

    get isSomethingSelected(): boolean {
        return this.filteredItems && this.filteredItems.some(x => x.isSelected);
    }

    get allSelected(): boolean {
        return !!this.filteredItems && this.filteredItems.every(x => x.isSelected);
    }
    set allSelected(value: boolean) {
        if (!!this.filteredItems)
            this.filteredItems.forEach(x => x.isSelected = value);
    }

    get elementWidth(): number {
        return this.$element.outerWidth();
    }

    private allItems: IGroupAssignmentModel[];

    constructor(private $scope: IScope,
                private $element: IAugmentedJQuery,
                private blockUI: blockUI.IBlockUIService,
                private messageService: IMessageService,
                private groupAssignmentService: IGroupAssignmentService,
                private confirmationService: IConfirmationService) {
        
        this.blockName = `accessRequestsToBeApprovedByCurrentUser_blockName`;
        this.filterModel = {
            groupsSubstring: '',
            membersSubstring: '',
            issuersSubstring: '',
            assignmentType: null,
                scope: {
                substring: '',
                isNull: null
            },
            timeRange: {
            }
        }
        this.setItemsAsync();
    }

    private setItemsAsync() {
        this.startBlock("Loading assignments...")

        const promise = this.groupAssignmentService.getGroupAssignmentsToBeApprovedByCurrentUser();

        return promise
            .then(groupAssignments => {
                this.allItems = groupAssignments;
            })
            .catch(() => {
                this.messageService.alertError(`Error on loading group assignments`);
                this.allItems = [];
            })
            .finally(() => {
                this.stopBlock();
                this.filterChanged();
            })
    }

    filterChanged() {
        if (this.filterForm.$valid) {
            const uiFilterModel: IRequestedGroupAssignmentsUiFilter = {
                assignmentType: this.filterModel.assignmentType,
                groupsSubstring: this.filterModel.groupsSubstring,
                membersSubstring: this.filterModel.membersSubstring,
                issuersSubstring: this.filterModel.issuersSubstring,
                from: this.filterModel.timeRange.from,
                to: this.filterModel.timeRange.to,
                isScopeNull: this.filterModel.scope.isNull,
                scopesSubstring: this.filterModel.scope.substring
            }

            this.filteredItems = this.groupAssignmentService.uiSideFilterGroupAssignments(this.allItems, uiFilterModel)
                .map(x => ({ ...x, isSelected: false }));
        }
    }

    approve() {
        if (this.isSomethingSelected) {
            this.confirmAction(`Please confirm that you want to approve selected assignments?`, `Error while trying to approving`, 'Approve', 'btn-success', () => {
                return this.groupAssignmentService.approveGroupAssignments(this.filteredItems.filter(x => x.isSelected).map(x => x.id));
            });
        }
    }

    reject() {
        if (this.isSomethingSelected) {
            this.confirmAction(`Please confirm that you want to reject selected assignments?`, `Error while trying to rejecting`, 'Reject', 'btn-danger', () => {
                return this.groupAssignmentService.rejectGroupAssignments(this.filteredItems.filter(x => x.isSelected).map(x => x.id));
            });
        }
    }

    private confirmAction(confirmationMessage: string, errorMessageTitle: string, okHtml: string, okClass: string, action: () => IPromise<any>) {
        this.confirmationService.open({
            cancelLabel: "Cancel",
            okLabel: okHtml,
            okCssClass: okClass,
            size: "lg",
            title: `Confirmation`,
            template: confirmationMessage
        }).then(() => {
            this.startBlock("Please wait...");
            action
            action()
                .then(() => {
                    this.stopBlock();
                    this.setItemsAsync();
                })
                .catch((errorMessage: string) => {
                    this.stopBlock();
                    this.messageService.alertError(`${errorMessageTitle}: ${errorMessage}.`);
                });
        });
    }

    private startBlock(message: string): void {
        this.blockUI.instances.get(this.blockName).start(message)
    }

    private stopBlock(): void {
        this.blockUI.instances.get(this.blockName).stop();
    }
}