'use strict';

import {
    register,
    IMessageService,
    IConfirmationService
} from '@systemorph/web';
import { IGroupAssignmentOfGroup, IGroupAssignmentOfGroupFilter, IGroupMetaInfo, IGroupAssignmentService } from "../../accessControl.api";
import { AC_COMMON_EVENT } from "../../accessControl.events";
import { AC_GROUP_ASSIGNMENT_MODE } from "../../accessControl.constants";
import { IFormController, IScope, blockUI, ITimeoutService, IAngularEvent, IPromise } from 'angular';

register.directive('groupWithoutScope', () => {
    return {
        replace: true,
        scope: true,
        template: require('./groupWithoutScope.html'),
        controller: GroupWithoutScopeController,
        controllerAs: 'gansCtrl',
        bindToController: {
            groupMetaInfo: "=",
            initialFilterText: "=",
            mode: "="
        }
    }
});

class GroupWithoutScopeController {
    //bindend
    groupMetaInfo: IGroupMetaInfo;
    initialFilterText: string;
    mode: string;

    filterForm: IFormController;

    currentGroupAssignments: IGroupAssignmentOfGroup[];
    showStatusFilter: boolean;
    blockName: string;
    assignmentStatus: string;
    filterText: string;

    constructor(private $scope: IScope,
                private $timeout: ITimeoutService,
                private messageService: IMessageService,
                private groupAssignmentService: IGroupAssignmentService,
                private confirmationService: IConfirmationService,
                private blockUI: blockUI.IBlockUIService) {
        
        this.blockName = `gansBlock_of_${this.groupMetaInfo.systemName}_with_at_${this.groupMetaInfo.assignmentType}`;
        this.showStatusFilter = this.mode === AC_GROUP_ASSIGNMENT_MODE.direct;
        this.assignmentStatus = null;
        this.filterText = /*this.initialFilterText || ''*/''; // no initial filter for groupAssignments without scopes

        this.setGroupAssignmentsAsync();

        this.$scope.$on(AC_COMMON_EVENT.deleteGroupAssignment, (e: IAngularEvent, assignment: IGroupAssignmentOfGroup) => {
            e.stopPropagation();

            this.confirmationService.open({
                cancelLabel: "Cancel",
                okLabel: "Ok",
                okCssClass: 'btn-danger',
                size: "lg",
                title: `Please confirm`,
                template: `Remove ${assignment.member.displayName} from ${this.groupMetaInfo.displayName}?`
            }).then(() => {
                this.startBlock("Deleting assignments...");
                this.groupAssignmentService.deleteGroupAssignments([assignment.id])
                    .then(() => {
                        this.stopBlock();
                        this.setGroupAssignmentsAsync();
                    })
                    .catch((errorMessage: string) => {
                        this.stopBlock();
                        this.messageService.alertError(`Error on deleting group assignments: ${errorMessage}.`);
                    });
            });
        })

        this.$scope.$on(AC_COMMON_EVENT.refreshGroupAssignment, (e: IAngularEvent) => {
            this.setGroupAssignmentsAsync();
        })
    }

    private currentLoadingPromise: IPromise<IGroupAssignmentOfGroup[]>;

    private setGroupAssignmentsAsync() {
        if (this.currentLoadingPromise == null) {
            this.startBlock("Loading assignments...");
        }

        const filter: IGroupAssignmentOfGroupFilter = {
            assignmentStatus: this.assignmentStatus,
            assignmentType: this.groupMetaInfo.assignmentType,
            groupId: this.groupMetaInfo.id,
            memberText: this.filterText,
            scopeIds: null,
            mode: this.mode
        }
        const promise = this.groupAssignmentService.getGroupAssignmentsOfGroup(filter);
        this.currentLoadingPromise = promise;
        return promise
            .then(groupAssignments => {
                if (this.currentLoadingPromise === promise) {
                    this.currentGroupAssignments = groupAssignments;
                }
            })
            .catch(() => {
                if (this.currentLoadingPromise === promise) {
                    this.messageService.alertError(`Error on loading group assignments`);
                    this.currentGroupAssignments = [];
                }
            })
            .finally(() => {
                if (this.currentLoadingPromise === promise) {
                    this.$timeout(() => this.stopBlock()); // due to odata request doesnt trigger digest sycle
                    this.currentLoadingPromise = null;
                }
            })
    }

    filterChanged() {
        if (this.filterForm.$valid) {
            this.setGroupAssignmentsAsync();
        }
    }

    private startBlock(message: string): void {
        this.blockUI.instances.get(this.blockName).start(message)
    }

    private stopBlock(): void {
        this.blockUI.instances.get(this.blockName).stop();
    }
}