'use strict';

import { register, IMessageService, IDateTimeService, DATE_TIME_KIND, DateTimeRangeModel } from '@systemorph/web';
import { IGroupAssignmentLogModelWithTotalCount, IGroupAssignmentService, IGroupAssignmentLogsBackendFilter } from '../../accessControl.api';
import {IScope, blockUI, IFormController, IPromise} from 'angular';

register.directive('groupAssignmentLogs', () => {
    return {
        replace: true,
        scope: true,
        restrict: 'E',
        template: require('./groupAssignmentLogs.html'),
        controller: GroupAssignmentLogsController,
        controllerAs: 'galCtrl',
        bindToController: {
            initialFilter: "=?",
            groupId: "=?",
            memberId: "=?",
            onFilterChanged: "&"
        }
    }
});

interface IFilterModel {
    timeRange: DateTimeRangeModel;
    groupsSubstring: string;
    membersSubstring: string;
    issuersSubstring: string;
    assignmentType?: string;
    activity: string;
    page: number;
    pageSize: number;
}

class GroupAssignmentLogsController {
    //bindend
    initialFilter: IGroupAssignmentLogsBackendFilter; 
    groupId: string; 
    memberId: string; 
    onFilterChanged: (args: { filter: IGroupAssignmentLogsBackendFilter }) => void;

    filterForm: IFormController;

    blockName: string;
    filterModel: IFilterModel;
    data: IGroupAssignmentLogModelWithTotalCount;

    readonly defaultPageSize = 20;
    readonly dateTimeKind: string = DATE_TIME_KIND.utc;
    readonly dateTimeFormat: string = 'short';

    constructor(private $scope: IScope,
                private blockUI: blockUI.IBlockUIService,
                private messageService: IMessageService,
                private groupAssignmentService: IGroupAssignmentService,
                private dateTimeService: IDateTimeService) {
        
        this.blockName = `groupAssignmentLogs_blockName`;
        const initialFilter = this.initialFilter || <any>{};
        const filterNormalized: IGroupAssignmentLogsBackendFilter = { 
            ...initialFilter, 
            from: this.dateTimeService.parse(initialFilter.from), 
            to: this.dateTimeService.parse(initialFilter.to), 
            pageSize: initialFilter.pageSize != null ? +initialFilter.pageSize : this.defaultPageSize  
        };
        this.filterModel = {
            timeRange: {
                from: filterNormalized.from,
                to: filterNormalized.to
            },
            groupsSubstring: filterNormalized.groupsSubstring,
            membersSubstring: filterNormalized.membersSubstring,
            issuersSubstring: filterNormalized.issuersSubstring,
            assignmentType: filterNormalized.assignmentType || null /*required for filter model binding*/,
            activity: filterNormalized.activity,
            page: filterNormalized.page != null? +filterNormalized.page + 1 : 1,
            pageSize: filterNormalized.pageSize
        }
        this.setDataAsync(filterNormalized);
    }

    private currentLoadingPromise: IPromise<IGroupAssignmentLogModelWithTotalCount>;

    private setDataAsync(filter: IGroupAssignmentLogsBackendFilter) {
        if (this.currentLoadingPromise == null) {
            this.blockUI.instances.get(this.blockName).start("Loading assignments...")
        }

        const promise = this.groupAssignmentService.getGroupAssignmentLogs(this.groupId, this.memberId, filter);
        this.currentLoadingPromise = promise;
        return promise
            .then(groupAssignments => {
                if (this.currentLoadingPromise === promise) {
                    this.data = groupAssignments;
                }
            })
            .catch(() => {
                if (this.currentLoadingPromise === promise) {
                    this.messageService.alertError(`Error on loading group assignment logs`);
                    this.data = {
                        logs: [],
                        totalCount: 0
                    };
                }
            })
            .finally(() => {
                if (this.currentLoadingPromise === promise) {
                    this.blockUI.instances.get(this.blockName).stop();
                    this.currentLoadingPromise = null;
                }
            })
    }

    filterChanged() {
        if (this.filterForm.$valid) {
            const filterModel: IGroupAssignmentLogsBackendFilter = {
                assignmentType: this.filterModel.assignmentType,
                groupsSubstring: this.filterModel.groupsSubstring,
                issuersSubstring: this.filterModel.issuersSubstring,
                membersSubstring: this.filterModel.membersSubstring,
                activity: this.filterModel.activity,
                page: this.filterModel.page != null ? this.filterModel.page - 1 : 0,
                pageSize: this.filterModel.pageSize,
                from: this.filterModel.timeRange.from,
                to: this.filterModel.timeRange.to,
            }

            this.setDataAsync(filterModel);

            if (this.onFilterChanged) {
                this.onFilterChanged({ filter: filterModel})
            }
        }
    }
}