'use strict';

import {
    register,
    IDateTimeService
} from "@systemorph/web";
import { IPromise, IHttpService } from 'angular';
import {
    IGroupAssignmentService,
    IGroupAssignmentOfGroup,
    IRequestedGroupAssignmentsUiFilter,
    IGroupAssignmentLogsBackendFilter,
    IGroupAssignmentLogModelWithTotalCount,
    IAccessObjectUiFriendlyModel,
    IGroupAssignmentModel,
    IGroupAssignmentOfGroupFilter,
    ICreateGroupAssignmentSummaryResult
} from "./accessControl.api";

class GroupAssignmentService implements IGroupAssignmentService {
    private readonly maxScopeIdsCount = 10;
    private readonly baseApiUrl: string = '/api/groupAssignmentUtils/';

    constructor(private $http: IHttpService, private dateTimeService: IDateTimeService) {
    }

    getGroupAssignmentsOfGroup(filter: IGroupAssignmentOfGroupFilter): IPromise<IGroupAssignmentOfGroup[]> {
        return this.$http.get<IGroupAssignmentOfGroup[]>(`${this.baseApiUrl}GroupAssignmentsOfGroup`, { params: filter })
            .then(result => result.data);
    }

    getGroupAssignmentLogs(groupId: string, memberId: string, filter: IGroupAssignmentLogsBackendFilter): IPromise<IGroupAssignmentLogModelWithTotalCount> {
        const params: any = { ...filter, groupId: groupId, memberId: memberId }
        return this.$http.get<IGroupAssignmentLogModelWithTotalCount>(`${this.baseApiUrl}groupAssignmentLogs`, { params: params })
            .then(result => {
                return {
                    totalCount: result.data.totalCount,
                    logs: <IGroupAssignmentModel[]>this.parseTimestamp(result.data.logs)
                };
            });
    }

    getGroupAssignmentsToBeApprovedByCurrentUser(): IPromise<IGroupAssignmentModel[]> {
        return this.$http.get<IGroupAssignmentModel[]>(`${this.baseApiUrl}groupAssignmentsToBeApprovedByCurrentUser`, { })
            .then(result => this.parseTimestamp(result.data));
    }

    getCurrentUserRequestedGroupAssignments(): IPromise<IGroupAssignmentModel[]> {
        return this.$http.get<IGroupAssignmentModel[]>(`${this.baseApiUrl}currentUserRequestedGroupAssignments`, { })
            .then(result => this.parseTimestamp(result.data));
    }

    private parseTimestamp(data: IGroupAssignmentModel[]) {
        data.forEach(x => {
            x.timestamp = this.dateTimeService.parse(x.timestamp);
        })
        return data;
    }

    isTooMuchScopesForRequest(scopeIds: string[]): boolean {
        return scopeIds && scopeIds.length > this.maxScopeIdsCount;
    }

    uiSideFilterGroupAssignments(groupAssignments: IGroupAssignmentModel[], filter: IRequestedGroupAssignmentsUiFilter): IGroupAssignmentModel[] {
        let ret = groupAssignments;

        ret = this.filterGroupAssignmentsByAccessObject(ret, x => x.member, filter.membersSubstring);
        ret = this.filterGroupAssignmentsByAccessObject(ret, x => x.group, filter.groupsSubstring);
        ret = this.filterGroupAssignmentsByAccessObject(ret, x => x.issuer, filter.issuersSubstring);
        
        if (filter.isScopeNull != null) {
            ret = ret.filter(x => x.scope != null && !filter.isScopeNull || x.scope == null && filter.isScopeNull);
        }
        if (!filter.isScopeNull /*isScopeNull != true*/) {
            ret = this.filterGroupAssignmentsByAccessObject(ret, x => x.scope, filter.scopesSubstring);
        }
        if (filter.assignmentType) {
            ret = ret.filter(x => x.assignmentType === filter.assignmentType);
        }
        if (filter.from) {
            ret = ret.filter(x => x.timestamp && x.timestamp >= filter.from);
        }
        if (filter.to) {
            ret = ret.filter(x => x.timestamp && x.timestamp <= filter.to);
        }
        return ret.slice();
    }

    private filterGroupAssignmentsByAccessObject(groupAssignments: IGroupAssignmentModel[],
                                                 getter: (x: IGroupAssignmentModel) => IAccessObjectUiFriendlyModel,
                                                 substring: string): IGroupAssignmentModel[] {
        const substringLower = substring ? substring.toLowerCase() : null;
        if (substringLower) {
            return groupAssignments.filter(x => {
                const value: IAccessObjectUiFriendlyModel = getter(x);

                return value && (value.displayName.toLowerCase().indexOf(substringLower) !== -1 || value.systemName.toLowerCase().indexOf(substring) !== -1)
            });
        }
        return groupAssignments;
    }
         

    createGroupAssignments(groupSystemNames: string[], assignmentType: string, memberSystemNames: string[], scopeIds?: string[]): IPromise<ICreateGroupAssignmentSummaryResult> {
        const payload = {
            groupSystemNames: groupSystemNames,
            assignmentType: assignmentType,
            memberSystemNames: memberSystemNames,
            scopeIds: scopeIds
        };
        return this.performGroupAssignmentsPostAction<ICreateGroupAssignmentSummaryResult>(`${this.baseApiUrl}createGroupAssignments`, payload);
    }

    deleteGroupAssignments(groupAssignmentIds: string[]): IPromise<any> {
        return this.performGroupAssignmentsPostAction<any>(`${this.baseApiUrl}deleteGroupAssignments`, { ids: groupAssignmentIds });
    }
    approveGroupAssignments(groupAssignmentIds: string[]): IPromise<any> {
        return this.performGroupAssignmentsPostAction<any>(`${this.baseApiUrl}approveGroupAssignments`, { ids: groupAssignmentIds });
    }
    rejectGroupAssignments(groupAssignmentIds: string[]): IPromise<any> {
        return this.performGroupAssignmentsPostAction<any>(`${this.baseApiUrl}rejectGroupAssignments`, { ids: groupAssignmentIds });
    }
    private performGroupAssignmentsPostAction<T>(url: string, payload: any): IPromise<T> {
        return this.$http.post<T>(url, payload, {})
            .then(result => result.data)
            .catch(result => {
                throw result.data ? result.data.message : "";
            });
    }
}

register.service("groupAssignmentService", GroupAssignmentService)