import "angular-ui-router";
import 'angular-resource';
import 'angular-route';

import { register, IMessageService, ITypeLayout } from "@systemorph/web";
import { ILogOnModel, IChangePasswordModel, ILogOnResponse, IAccountService, IAccountResponse } from '../app/account/account.api';
import { IWindowService, module, mock, IModule, IHttpBackendService, IPromise } from "angular";

interface ITestAccountService extends IAccountService {
    $location: Location;
    $window: IWindowService;
    layout(url: string): IPromise<ITypeLayout>;
}

import '../app/account/accountService';

describe('AccountService', () => {
    let moduleName = 'testAccountServiceModule';
    let app: IModule;

    interface ILocationMock extends Location {
        path(newPath: string): ILocationMock;
    }

    // Mocks and Stubs
    let locationMock: ILocationMock;
    let windowMock: IWindowService;

    beforeAll(() => {
        app = module(moduleName, ['ngRoute', 'ngResource', 'ui.router', register.mainModuleName]);

        app.service('dataSourceService', () => { 
            return {
                getHeaders: jest.fn(),
                getDataSourceAsString: jest.fn().mockReturnValue("persistenceContext=TestContext")
            }            
        });
        app.service('messageService', () => {
            return {
                alertError: jest.fn()
            }
        });
        app.service('signalRService', () => {return {} });

        locationMock = <ILocationMock>{};
        locationMock.path = (path: string) => {
            locationMock.href = path;
            locationMock.replace = () => {};
            return locationMock;
        };
        locationMock.href = "deadbeef";

        windowMock = <IWindowService>{};
        (<any>windowMock).location = locationMock;
        windowMock.location.href = "deadbeef";
    });

    beforeEach(
        mock.module(moduleName)
    );

    let _accountService: ITestAccountService;
    let _messageService: IMessageService;
    let _httpBackend: any;

    beforeEach(inject((accountService: ITestAccountService,
                        messageService: IMessageService,
                        $httpBackend: IHttpBackendService) => {
        
        _accountService = accountService;
        _messageService = messageService;
        _httpBackend = $httpBackend;

        spyOn(_messageService, 'alertError').and.callFake(() => {});
    }));

    afterEach(() => {
        if (_httpBackend) {
            _httpBackend.verifyNoOutstandingExpectation(false);
            _httpBackend.verifyNoOutstandingRequest();
            _httpBackend.resetExpectations();
        }
    });

    it("IncorrectUserLogonShouldFail", () => {
        expect.assertions(3);

        const errorText = "Test error text";
        const requestData: ILogOnModel = { UserName: "invalidUser", Password: "testPassword", RememberMe: false, ReturnUrl: "testPath" };
        const responseData: ILogOnResponse = { success: false, error: errorText, userName: null };

        _httpBackend.expectPOST("/api/account/logon", requestData).respond(responseData);
        _accountService.$location = locationMock;
        _accountService.$window = windowMock;

        try {
            _accountService.logonUser(requestData);
            _httpBackend.flush();
        }
        catch (e) {
            expect(e.success).toBeFalsy();
            expect(windowMock.location.href).not.toBe(requestData.ReturnUrl); // "Should not redirect to redirectUrl";
            expect(_messageService.alertError).toBeCalledWith(errorText); // "Should call alertError with Error text from response"
        }
    });

    it("LogonCorrectUserShouldRedirect", () => {
        expect.assertions(2);

        const requestData: ILogOnModel = { UserName: "validUser", Password: "testPassword", RememberMe: false, ReturnUrl: "testPath" };
        const responseData: ILogOnResponse = <ILogOnResponse>{ success: true };
        _accountService.$location = locationMock;
        _accountService.$window = windowMock;

        _httpBackend.expectPOST("/api/account/logon", requestData).respond(responseData);

        _accountService.logonUser(requestData).then((data: ILogOnResponse) => {
            expect(windowMock.location.href).toBe(requestData.ReturnUrl); // "Should redirect to returnUrl"
            expect(_messageService.alertError).not.toBeCalled(); // "alertError shouldn't be called";
        });

        _httpBackend.flush();
    });

    it("LogonCorrectUserShouldRedirectToDefaultUrlIfNoReturnUrlProvided", () => {
        expect.assertions(2);
    
        var requestData = <ILogOnModel> { UserName: "validUser", Password: "testPassword", RememberMe: false };
        var responseData = { success: true };
        _accountService.$location = locationMock;
        _accountService.$window = windowMock;
    
        _httpBackend.expectPOST("/api/account/logon", requestData).respond(responseData);

        _accountService.logonUser(requestData).then((data: ILogOnResponse) => {
            expect(windowMock.location.href).toBe("/"); // "Should redirect to '/'"
            expect(_messageService.alertError).not.toBeCalled(); // "alertError shouldn't be called";
        });

        _httpBackend.flush();
    });    

    it("SuccessfulChangePassword", () => {
        expect.assertions(2);

        const returnUrl = "testPath";
        const requestData: IChangePasswordModel = { OldPassword: "testPasswordOld", NewPassword: "testPasswordNew", ConfirmPassword: "testPasswordNew" };
        const responseData: IAccountResponse = { success: true, error: null };
        _accountService.$location = locationMock;
        _accountService.$window = windowMock;

        _httpBackend.expectPOST("/api/account/changePassword", requestData).respond(responseData);

        _accountService.changePassword(requestData, returnUrl).then((data: IAccountResponse) => {
            expect(locationMock.href).toBe(returnUrl); // "Successful password change leads to redirection";
            expect(_messageService.alertError).not.toBeCalled(); // "alertError shouldn't be called";
        });

        _httpBackend.flush();
    });

    it("SuccessfulChangePasswordWithNoReturnUrlProvided", () => {
        expect.assertions(2);

        const requestData: IChangePasswordModel = { OldPassword: "testPasswordOld", NewPassword: "testPasswordNew", ConfirmPassword: "testPasswordNew" };
        const responseData: IAccountResponse = { success: true, error: null };
        _accountService.$location = locationMock;
        _accountService.$window = windowMock;

        _httpBackend.expectPOST("/api/account/changePassword", requestData).respond(responseData);
    
        _accountService.changePassword(requestData, null).then((data: IAccountResponse) => {
            expect(locationMock.href).toBe("/"); // "Successful password change leads to redirection";
            expect(_messageService.alertError).not.toBeCalled(); // "alertError shouldn't be called";
        });

        _httpBackend.flush();
    });

    it("ChangePasswordFailed", () => {
        expect.assertions(3);

        const returnUrl = "testPath";
        const error = "textError";
        const requestData: IChangePasswordModel = { OldPassword: "testPasswordOld", NewPassword: "testPasswordNew", ConfirmPassword: "testPasswordNew" };
        const responseData: IAccountResponse = { success: false, error: error };
        _accountService.$location = locationMock;
        _accountService.$window = windowMock;

        _httpBackend.expectPOST("/api/account/changePassword", requestData)
        .respond(responseData);

        try {
            _accountService.changePassword(requestData, returnUrl);
            _httpBackend.flush();
        }
        catch (e) {
            expect(e.success).toBeFalsy();
            expect(locationMock.href).not.toBe(returnUrl); // "Unsuccessful password change shouldn't leads to redirection";
            expect(_messageService.alertError).toHaveBeenCalledWith(error); // "Should call alertError with Error text from response";
        }
    });

    it("LogOnLayout", () => {
        expect.assertions(1);

        _accountService.layout = jest.fn();

        _accountService.logOnLayout();

        expect(_accountService.layout).toBeCalledWith('/api/account/logOnLayout');
    });

    it("ChangePasswordLayout", () => {
        expect.assertions(1);

        _accountService.layout = jest.fn();

        _accountService.changePasswordLayout();

        expect(_accountService.layout).toBeCalledWith('/api/account/changePasswordLayout');
    });

    it("Layout", () => {
        expect.assertions(1);
       
        const url = '/some/path/to/something';
        const responseData = { someField: "someValue" };

        _httpBackend.expectGET(url).respond(responseData);

        _accountService.layout(url).then((data: any) => {
            expect(data).toEqual(responseData);
        });

        _httpBackend.flush();
    });
});